#ifndef TABLEAU_H
#define TABLEAU_H
#include "minit.h"


struct TABLEAU {
  int maxcolumns, maxrows;		/* To keep track of array sizes. */

  /*
    no_constraints + no_vars - no_eq_constraints + 1
   */
  int columns;
  int rows;

  int no_eq;
  int no_ineq;
  int no_cons;
  int no_vars;
  int no_neg_rhs;
  int no_neg_costcoef;

  /* these   arrays are TABLEAU_COLS long */
  Real *thmin;

  /*
    neg_costcoef_indices == { j |  elements[0][j] < 0 }

    iow, the columns in which the cost coefficient is negative.

  */

  int *neg_costcoef_indices;
  int *imin;

  /* these  arrays are TABLEAU_ROWS long */
  int *jmax;


  /* neg_rhs == { i  |      elements[i][columns-1]  < 0},

     in other words,  the set of constraints in which the RHS is  negative.  */
  
  int *neg_rhs;
  int *chk;
  Real *delmax;
  
  /* elements is a matrix with with ROWS rows and COLS columns, (where
        TABLEAU_COLS = m + n - no_eq_constraints + 1, TABLEAU_ROWS = m
        + 1) and forms the initial tableau of the algorithm.  */
  
  Real **elements;

  int   im, jm, jmin, imax;
  Real gmin;
  Real phimax;
};


struct TABLEAU *new_tableau(void);
void free_tableau (struct TABLEAU *tp);
void init_tableau (struct TABLEAU *tabp);
void print_tableau(struct TABLEAU *tabp);
void set_tableau_for_problem (struct TABLEAU *t, struct LP_PROBLEM *lp);
void tableau_set_max_columns (struct TABLEAU *t, int maxcolumns);
void tableau_set_max_rows (struct TABLEAU*t, int maxrows);
void tableau_set_rows_columns (struct TABLEAU *t, int r, int c);
void tableau_read_solution (struct TABLEAU *tabp, struct LP_SOLUTION *sol);
#endif
